#!/bin/bash

WORKING_DIR=$PWD

MODE=$1

ARG_MCG_HOST_PORT=
ARG_KEY=
ARG_LOCAL_PORT=4433

ENABLE_CLUSTER=


function print_enable_help(){
	echo "		$0 enable --mcg-host=[ip:port] --shared-key=[key] --local-port=[port]"
	echo
	echo "			--mcg-host      zCenter host. Ex: 192.168.1.1:443"
	echo 
	echo "			--shared-key    Key for authentication. You can get it from zCenter when ZNS node is created."
 	echo
 	echo "			--local-port    local port which listens for accepting connection from zCenter."
	echo "					(default to 4433)"
	echo
}


function print_disable_help(){
	echo "		$0 disable"
	echo
}


function print_mode_help(){
	echo
        echo "Incorrect syntax."
        echo "	For enable:"
        print_enable_help
	echo
	echo "	For disable:"
	print_disable_help
}


function parse_enable_args(){
	for i in "$@"; do
		case $i in
			--mcg-host=*)
				ARG_MCG_HOST_PORT="${i#*=}"
				shift
				;;
			--shared-key=*)
				ARG_KEY="${i#*=}"
				shift
				;;
			--local-port=*)
				ARG_LOCAL_PORT="${i#*=}"
				shift
				;;
			--*)
				echo "Unknown option $i"
				exit 1
				;;
			*)
				;;
		esac
	done
}


function print_enable_args(){
        echo
        echo ================================================================
	echo
	echo "Going to enable cluster with:"
        echo
        echo "	MCG Host    = $ARG_MCG_HOST_PORT"
        echo "	Shared key  = $ARG_KEY"
        echo "	Local port  = $ARG_LOCAL_PORT"
        echo
        echo =================================================================
}


function check_zns(){
	rpm -qa |grep zns-sg > /dev/null 2>&1
	if [ $? -ne 0 ]; then
		echo
		echo "ZNS is not installed."
		echo
		exit 1
	fi
}

function check_enable_args(){
        if [ -z $ARG_MCG_HOST_PORT ]; then
                echo 
                echo "No MCG host info. "
                echo
		print_enable_help
                return
        fi

        if [ -z $ARG_KEY ];then
                echo
                echo "No shared key."
                echo
		print_enable_help
                return
        fi

        ENABLE_CLUSTER=1
	print_enable_args
}


function enable_cluster(){
        local znscfg="/zentera/bin/network-daemon/zns.cfg"
        if [ -f $znscfg ]; then
                if grep -q "^shared_key=" $znscfg; then
                        echo "Cluster has already been enabled on this gateway"
                        exit 1
                fi
        fi

        echo "  download certificaste..."
        rm -rf /zentera/download/getcacert?z=$ARG_KEY
        r=`wget -P /zentera/download --tries=3 --timeout=10 --no-check-certificate -q https://$ARG_MCG_HOST_PORT/getcacert?z=$ARG_KEY`
        if [ $? -ne 0 ]; then
                echo "Downloading zns certificate files from MCG failed. Quit!"
                exit 1
        fi
        unzip -q -o /zentera/download/getcacert?z=$ARG_KEY  -d /zentera/bin/network-daemon

        echo "  configure key..."
        if [ ! -f $znscfg ]; then
                echo "shared_key=$ARG_KEY" > $znscfg
        else
                if grep -R "shared_key" $znscfg > /dev/null; then
                        sed -i "/shared_key=/c\shared_key=$ARG_KEY" $znscfg
                else
                        echo "shared_key=$ARG_KEY" >> $znscfg
                fi
        fi

        echo "  configure local port..."
        if test "$ARG_LOCAL_PORT" != ""; then
                if grep -R "cg_port" $znscfg > /dev/null; then
                        sed -i "/cg_port=/c\cg_port=$ARG_LOCAL_PORT" $znscfg
                else
                        echo "cg_port=$ARG_LOCAL_PORT" >> $znscfg
                fi
        fi

        echo "  set iptables..."
        local port=`grep  "^cg_port=" $znscfg | cut -f2 -d"=" | head -1`
        if [ -z "$port" ]; then
                iptables -I INPUT -p tcp -m state --state NEW -m tcp --dport 4433 -j ACCEPT
        else
                iptables -I INPUT -p tcp -m state --state NEW -m tcp --dport $port -j ACCEPT
        fi
        service iptables save > /dev/null

        chmod 600 $znscfg
        /etc/init.d/zns-sgd restart >/dev/null
        chkconfig zns-sgd on > /dev/null
        rm -rf /zentera/download/getcacert?z=$ARG_KEY
        echo "  done."
}


function disable_cluster(){
#	while true; do
#		if [ "${ZCLI_SILENT}" = "yes" ]; then
#			yn="yes"
#		else
#			read -p "Are you sure you want to disable?(yes/no) " yn
#		fi
#		case $yn in
#			[Yy]* )
				local znscfg="/zentera/bin/network-daemon/zns.cfg"
				port=`grep  "^cg_port=" $znscfg | cut -f2 -d"=" | head -1`
				if [ -z "$port" ]; then
					iptables -D INPUT -p tcp -m state --state NEW -m tcp --dport 4433 -j ACCEPT
				else
					iptables -D INPUT -p tcp -m state --state NEW -m tcp --dport $port -j ACCEPT
				fi
				service iptables save > /dev/null

				sed -i '/shared_key=/d' $znscfg
				chkconfig zns-sgd off > /dev/null
				/etc/init.d/zns-sgd stop

				rm -rf /zentera/bin/network-daemon/.allowed-cgs
				rm -rf /zentera/bin/network-daemon/znscacert.*

#				break;;
#			[Nn]* ) exit;;
#				* ) echo "Please answer yes or no.";
#		esac
#      done
}


check_zns

if [ "$MODE" == "enable" ]; then
	shift
        parse_enable_args $@
	check_enable_args
	enable_cluster
elif [ "$MODE" == "disable" ]; then
	disable_cluster
elif [ "$MODE" == "--help" ]; then
        print_mode_help
        exit 0
else
        print_mode_help
        exit 1
fi

